function balanceStructure=getElementalBalance(model,rxns,printUnbalanced,printUnparsable)
% getElementalBalance
%   Checks a model to see if the reactions are elementally balanced
%
%   model             a model structure
%   rxns              either a cell array of reaction IDs, a logical vector 
%                     with the same number of elements as reactions in the model,
%                     of a vector of indexes. Only these reactions will be
%                     checked (opt, default model.rxns)
%   printUnbalanced   print warnings about the reactions that were 
%                     unbalanced (opt, default false)
%   printUnparsable   print warnings about the reactions that cannot be
%                     parsed (opt, default false)
%
%   balanceStructure
%       balanceStatus	1 if the reaction is balanced, 0 if it's unbalanced, 
%                      -1 if it couldn't be balanced due to missing information, 
%                      -2 if it couldn't be balanced due to an error
%       leftComp        An array with the sum of coefficients for carbon, phosphate, 
%                       sulfur, nitrogen, "R-groups", oxygen, hydrogen for the left side
%       rightComp       The corresponding array for the right side
%
%   Usage: balanceStructure=getElementalBalance(model,rxns,printUnbalanced,printUnparsable)
%
%   Rasmus Agren, 2013-02-20
%

if nargin<2
    rxns=[];
end

if nargin<3
    printUnbalanced=false;
end

if nargin<4
    printUnparsable=false;
end

if ~isempty(rxns)
   indexes=~getIndexes(model,rxns,'rxns',true);
   model=removeRxns(model,indexes);
end

%Get the formulas. Formulas should be parsed from the InChIs when defined,
%otherwise the metFormulas values are used.
if isfield(model,'metFormulas')
    %Get the elemental composition of all metabolites
    [nC, nP, nS, nN, nR, nO, nH, exitFlag]=parseFormulas(model.metFormulas,true);
else
    nC=zeros(numel(model.mets),1);
    nP=nC;
    nS=nC;
    nN=nC;
    nR=nC;
    nO=nC;
    nH=nC;
    exitFlag=nC;
end
if isfield(model,'inchis')
    [nCI, nPI, nSI, nNI, nRI, nOI, nHI, exitFlagI]=parseFormulas(model.inchis,true,true);
    
    %Use the values calculated from the InChI codes if successful
    nC(exitFlagI==1)=nCI(exitFlagI==1);
    nP(exitFlagI==1)=nPI(exitFlagI==1);
    nS(exitFlagI==1)=nSI(exitFlagI==1);
    nN(exitFlagI==1)=nNI(exitFlagI==1);
    nR(exitFlagI==1)=nRI(exitFlagI==1);
    nO(exitFlagI==1)=nOI(exitFlagI==1);
    nH(exitFlagI==1)=nHI(exitFlagI==1);
    exitFlag(exitFlagI==1)=exitFlagI(exitFlagI==1);
end

balanceStructure.balanceStatus=zeros(numel(model.rxns),1);
balanceStructure.leftComp=zeros(numel(model.rxns),7);
balanceStructure.rightComp=zeros(numel(model.rxns),7);

%Loop through each reaction and calculate the total amount of each compound
for i=1:numel(model.rxns)
    leftMets=find(model.S(:,i)<0);
    rightMets=find(model.S(:,i)>0);
    
    %Calculate the composition of left/right side even if the reaction as a
    %whole cannot be balanced
    balanceStructure.leftComp(i,1)=abs(sum(model.S(leftMets,i).*nC(leftMets)));
    balanceStructure.leftComp(i,2)=abs(sum(model.S(leftMets,i).*nP(leftMets)));
    balanceStructure.leftComp(i,3)=abs(sum(model.S(leftMets,i).*nS(leftMets)));
    balanceStructure.leftComp(i,4)=abs(sum(model.S(leftMets,i).*nN(leftMets)));
    balanceStructure.leftComp(i,5)=abs(sum(model.S(leftMets,i).*nR(leftMets)));
    balanceStructure.leftComp(i,6)=abs(sum(model.S(leftMets,i).*nO(leftMets)));
    balanceStructure.leftComp(i,7)=abs(sum(model.S(leftMets,i).*nH(leftMets)));

    balanceStructure.rightComp(i,1)=abs(sum(model.S(rightMets,i).*nC(rightMets)));
    balanceStructure.rightComp(i,2)=abs(sum(model.S(rightMets,i).*nP(rightMets)));
    balanceStructure.rightComp(i,3)=abs(sum(model.S(rightMets,i).*nS(rightMets)));
    balanceStructure.rightComp(i,4)=abs(sum(model.S(rightMets,i).*nN(rightMets)));
    balanceStructure.rightComp(i,5)=abs(sum(model.S(rightMets,i).*nR(rightMets)));
    balanceStructure.rightComp(i,6)=abs(sum(model.S(rightMets,i).*nO(rightMets)));
    balanceStructure.rightComp(i,7)=abs(sum(model.S(rightMets,i).*nH(rightMets)));
    
    %Are they ok?
    if all(exitFlag([leftMets; rightMets]))
        if all(exitFlag([leftMets; rightMets])==1)             
             %Check if it's balanced (use a cut off for fractional
             %coefficients)
             if sum(abs(balanceStructure.leftComp(i,:)-balanceStructure.rightComp(i,:)))<10^-6
                 balanceStructure.balanceStatus(i)=1;
             end
        else
            balanceStructure.balanceStatus(i)=-2;
        end
    else
        balanceStructure.balanceStatus(i)=-1;
    end
end

%Print warnings
toPrint=[];
if printUnbalanced==true
   toPrint=[toPrint;find(balanceStructure.balanceStatus==0)];
end
if printUnparsable==true
   toPrint=[toPrint;find(balanceStructure.balanceStatus<0)];
end

compounds={'carbon' 'phosphate' 'sulfate' 'nitrogen' 'R-groups' 'oxygen' 'hydrogen'};
toPrint=sort(toPrint);
for i=1:numel(toPrint)
    if balanceStructure.balanceStatus(toPrint(i))<0
    	fprintf(['WARNING: The reaction ' model.rxns{toPrint(i)} ' could not be parsed\n']);
    else
       %Find the compounds that it's not balanced for
       notBalanced=find(abs(balanceStructure.leftComp(toPrint(i),:)-balanceStructure.rightComp(toPrint(i),:))>10^-6);
       for j=1:numel(notBalanced)
            fprintf(['WARNING: The reaction ' model.rxns{toPrint(i)} ' is not balanced with respect to ' compounds{notBalanced(j)} '\n']);
       end
    end
end

end
